/**
 * Collection of current offers accepted by the user and proposed ranstad composed of ListedOfferModel elements
 */
define(['jquery',
'backbone',
'pamhelper',
'models/ListedOfferModel',
'dateHelper',
'underscore'
],function($,Backbone,pamHelper,ListedOfferModel,DateHelper,_){
	var OffersCollection = Backbone.Collection.extend({
		model: ListedOfferModel,
		ClearCollection : function(){
			_.each(_.clone(this.models), function(modelToDestroy) {
				modelToDestroy.destroy();
			});
		},
		GetCurrentOffers: function() {
			return this.filter(function(offer){ return offer.get("Statut") == "en cours"});
		},
		GetCurrentUnreadOffersTag: function() {
			return this.filter(function(offer){ return offer.get("Lu") == false && offer.get("Statut") == "nouveau"});
		},
		GetCurrentUnreadOffers: function() {
			return this.filter(function(offer){ return offer.get("Lu") == false});
		},
		
		GetSuggestedOffers: function() {
			return this.without.apply(this, this.GetCurrentOffers());
		},
		dataInCache : function(){
			var cachedValue = MyApp.oSessionManager.getCacheValue('mission');
			if(cachedValue){
			
				return cachedValue;
			}
			
			return undefined;
		},
		storeDataInCache : function(data){
			MyApp.oSessionManager.setCacheValue("mission",JSON.stringify(data));
		},
		removeDataFromCache : function(){
			storeDataInCache(undefined);
		},
		callWebService : function(successCallback,errorCallback){
			var thisObject = this;
			var myAppObject = MyApp;
			pamHelper.sendRestRequest({
				url: 'Mission/',
				success: function (data) {
					wsResult = data;
					thisObject.initialiseData(thisObject,data);
					if(successCallback) {
						successCallback(thisObject);
					}

				},
				error: function(data, errorMessage){
					if(errorCallback) {
						errorCallback(data,errorMessage);
					}
				}

			});
		},
		initialiseData: function(callingObject,data) {
			var thisObject = this;
			thisObject.ClearCollection();
			thisObject.loaded = true;
			datehHelper = new DateHelper();
			for (var i = 0; i < data.length; i++){
				if (data[i].DateFin != undefined){
					data[i].DateFin = datehHelper.parseDate(data[i].DateFin);
					data[i].FinString = datehHelper.formatDate(data[i].DateFin);
				}
				if (data[i].DateDebut != undefined){
					data[i].DateDebut = datehHelper.parseDate(data[i].DateDebut);
					data[i].DebutString = datehHelper.formatDate(data[i].DateDebut);
				}
				if (data[i].DateStatut != undefined){
					data[i].DateStatut = datehHelper.parseDate(data[i].DateStatut);
					data[i].DateStatutString = datehHelper.formatDate(data[i].DateStatut);			  
				}
				thisObject.add(new ListedOfferModel(data[i]));
			}
			
			this.storeDataInCache(thisObject);
		},
		initialise: function(successCallback,errorCallback){
			this.callWebService(successCallback,errorCallback);
		}	
	});
	
	return OffersCollection; 
});