/**
 * A model for availability
 */
define([
'jquery',
'backbone',
'pamhelper'
],function($,Backbone,pamHelper){
	var AvailabilityModel = Backbone.Model.extend({
	    defaults: {
	    },
		dataInCache : function(annee, semaine){
			var cachedValue = MyApp.oSessionManager.getCacheValue('planning_'+ annee + semaine);
			if(cachedValue){
				
				console.log('data in cache');
				return JSON.parse(cachedValue);
			}
			console.log('data expired');
			return undefined;
		},
		storeDataInCache : function(annee, semaine, data){
			MyApp.oSessionManager.setCacheValue('planning_'+ annee + semaine,JSON.stringify(data));
		},
		callWebService : function(annee, semaine){
			var thisObject = this;
			var myAppObject = MyApp;
			pamHelper.sendRestRequest({
				url:  'Planning/' + annee +'/' + semaine,
				success: function (data) {
					thisObject.storeDataInCache(annee, semaine, data)
					thisObject.initialiseData(thisObject, data);
				}
			});
		},
		initialiseData: function(callingObject,data) {
			if (callingObject.attributes != undefined && callingObject.attributes.Plannings != undefined) {
				console.log('should add new planning ')
				// add the planning got from the web service to the beginning
				var attribute = callingObject.attributes.Plannings;
				if (attribute[0] != undefined && data.Plannings[0].Annee <= attribute[0].Annee && data.Plannings[0].Semaine < attribute[0].Semaine){
					attribute.unshift(data.Plannings[0]);
				}
				else {
					attribute.push(data.Plannings[0]);
				}
				callingObject.set(attribute);
			} else {
				callingObject.set(data);
			}
			callingObject.change();
		},
		setView: function(viewToSet){
			this.view = viewToSet;
		},
		initialize: function(args, annee, semaine){
			this.initialise(annee, semaine);
		},
		initialise: function(annee, semaine){
			data = this.dataInCache(annee, semaine);
			if(data){
				this.initialiseData(this,data);
			}
			else{
				this.callWebService(annee, semaine);
			}
		},
		commitWeek: function(indexPlanningToCommit, callbackOnSuccess, callbackOnError, callbackNothingToDo){
			var planningToCommit = this.attributes.Plannings[indexPlanningToCommit];
			if (this.isPlanningWeekModified(planningToCommit)) {
				var thisReference = this;
				var myAppObject = MyApp;
				pamHelper.sendRestRequest({
					url:  'Planning/' + planningToCommit.Annee + '/' + planningToCommit.Semaine,
					type:'PUT',
					data: JSON.stringify({ Plannings: [planningToCommit] }),
					success: function (data) {
						// Update the cache with the commited week
						thisReference.storeDataInCache(planningToCommit.Annee, planningToCommit.Semaine, data);
						thisReference.attributes.Plannings[indexPlanningToCommit] = data.Plannings[0];
						callbackOnSuccess(data);
					},
					error : function (data, errorMessage){
						callbackOnError(data, errorMessage);
					}
				});
			}
			else {
				callbackNothingToDo(null);
			}
		},
		// returns if the planning has been modified, planningIndex is the index of the planning currently displayed to the user
		isPlanningModified : function (planningIndex){
			// getting the planning currently displayed to the user
			var currentPlanning = this.attributes.Plannings[planningIndex];
			return this.isPlanningWeekModified(currentPlanning);
		},
		isPlanningWeekModified : function (planning){
			for (var day = 0; day < 7; day ++){
				for (var daytime = 0; daytime < 3; daytime ++){
					if (planning.Planning[day][daytime].Modifie){
						return true;
					}
				}
			}
			return false;
		}
	});
	
	return AvailabilityModel; 
});