/**
 * A offer model for enCoursDetails and propositionDetails view including the folling attributes :
 * SecteurActivité
 * ID : unique id of the offer
 * TitreOffre : title of the offer
 * LieuTravail : Localisation of the offer
 * TypeContrat : contract type proposed in the offer
 * Debut : Stating date of the offer
 * Fin : Ending date of the offer
 * Duree : duration of the contract
 * ExperienceRequise : experience required for the offer
 * Descriptif : text describing the offer including h1 markup for titles
 * SyntheseOffre : small text describing the offer
 * NomEntreprise : company's name
 * AddresseEntreprise : company's address
 * NomContact : name of the contact in the customer company
 * PosteContact : title of the contact in the customer company
 * TelephoneContact : phone number of the contact in the customer company
 * InformationsEntrepriseURL : url pointing to a pdf describing the company
 * LivretAccueilURL : url pointing to welcome information
 * RegleSecuriteURL : url pointing to a pdf listing security rules
 * PlanDAccessURL : url pointing to a pdf representing a map to access the company
 */
define([
'backbone','dateHelper','pamhelper'
],function(Backbone,DateHelper,pamHelper){
	var DetailedOfferModel = Backbone.Model.extend({
	    defaults: {
	    },
		dataInCache : function(idMission){
			var cachedValue = MyApp.oSessionManager.getCacheValue('details_'+idMission);
			if(cachedValue){
				
				console.log('data in cache');
				return JSON.parse(cachedValue);
			}
			console.log('data expired');
			return undefined;
		},
		storeDataInCache : function(idMission,data){
			MyApp.oSessionManager.setCacheValue('details_'+idMission,JSON.stringify(data));
		},
		callWebService : function(idMission){
			var thisObject = this;
			var myAppObject = MyApp;
			pamHelper.sendRestRequest({
				url:'Mission/' + idMission+'/Details',
				success: function (data) {
					if (data.DateDebut != undefined) {
						datehHelper = new DateHelper();
						data.DateDebut = datehHelper.parseDate(data.DateDebut);
						data.DebutString = datehHelper.formatDate(data.DateDebut);
					}
					thisObject.storeDataInCache(idMission,data)
					thisObject.initialiseData(thisObject,data);
				},
				error : function (data, errorMessage){ 
					window.location.hash ='#propositions?error='+errorMessage.LibelleErreur;
				}
			});
		},
		initialiseData: function(callingObject,data) {
			var thisObject = this;
			callingObject.set(data);
			callingObject.change();
		},
		initialize: function(options){
			
		},
		initialise: function(options){
			var idMission= options.idMission;
			data = this.dataInCache(idMission);
			if(data){
				this.initialiseData(this,data);
			}
			else{
				this.callWebService(idMission);
			}
		}
	});
	
	return DetailedOfferModel; 
});