/**
 * A offer model for enCoursDetails and propositionDetails view including the folling attributes :
 * SecteurActivité
 * ID : unique id of the offer
 * TitreOffre : title of the offer
 * LieuTravail : Localisation of the offer
 * TypeContrat : contract type proposed in the offer
 * Debut : Stating date of the offer
 * Fin : Ending date of the offer
 * Duree : duration of the contract
 * ExperienceRequise : experience required for the offer
 * Descriptif : text describing the offer including h1 markup for titles
 * SyntheseOffre : small text describing the offer
 * NomEntreprise : company's name
 * AddresseEntreprise : company's address
 * NomContact : name of the contact in the customer company
 * PosteContact : title of the contact in the customer company
 * TelephoneContact : phone number of the contact in the customer company
 * InformationsEntrepriseURL : url pointing to a pdf describing the company
 * LivretAccueilURL : url pointing to welcome information
 * RegleSecuriteURL : url pointing to a pdf listing security rules
 * PlanDAccessURL : url pointing to a pdf representing a map to access the company
 */
define([
'backbone','dateHelper','pamhelper'
],function(Backbone, DateHelper,pamHelper){
	var DetailedOfferModel = Backbone.Model.extend({
		defaults: {
		},
		dataInCache : function(options){
			var cachedValue = MyApp.oSessionManager.getCacheValue('details_'+options.id);
			if(cachedValue){
				console.log('data in cache');
				return JSON.parse(cachedValue);
			}
			console.log('data expired');
			return undefined;
		},
		storeDataInCache : function(options,data){
			MyApp.oSessionManager.setCacheValue('details_'+options.id,JSON.stringify(data));
		},
		callWebService : function(options){
			var thisObject = this;
			var myAppObject = MyApp;
			pamHelper.sendRestRequest({
				url: 'Mission/' + options.id+'/Details',
				success: function (data) {
					if (data.DateDebut != undefined) {
						data.DateDebut = new Date(parseInt(data.DateDebut.substr(6)));
						data.DebutString = new DateHelper().formatDate(data.DateDebut);
					}
					thisObject.storeDataInCache(options,data)
					thisObject.initialiseData(thisObject,data);
				},
				error : function (data,errorMessage){
					window.location.hash ='#propositions?error='+errorMessage.LibelleErreur;
				}
			});
		},
		initialiseData: function(callingObject,data) {
			var thisObject = this;
			thisObject.set(data);
			thisObject.change();
		},
		initialize: function(options){
			
		},
		initialise: function(options){
			console.log('initialize DetailedProposition');
			var data = this.dataInCache(options);
			if(data){
				this.initialiseData(this,data);
			}
			else{
				this.callWebService(options);
			}
			
		},
		
		refuser : function(reason) {
			var thisObject = this;
			var myAppObject = MyApp;
			pamHelper.sendRestRequest({
				url: 'mission/'+thisObject.get('Id')+'/refuser',
				type: 'POST',
				data: '{ Motif : "' + reason + '"}',
				success: function (data) {
					MyApp.oSessionManager.clearCacheValue("mission");
					if (reason == 'contrainte'){
						window.location.hash = '#contact';
					}
					else if (reason == 'indisponibilite'){
						var dateDebutIndispo = null;
						for(var i=0; i < thisObject.attributes.Descriptif.length; i++){
							if(thisObject.attributes.Descriptif[i].Libelle == "Début"){
								dateDebutIndispo = thisObject.attributes.Descriptif[i].Contenu;
							}
						}
						if(dateDebutIndispo != null){
							window.location.hash = '#indisponibilites/' + new DateHelper().getDateForUrlPassing(dateDebutIndispo);
						}else{
							window.location.hash = '#indisponibilites';
						}
					}
					else {
						window.location.hash = '#propositions';
					}
				},
				error : function (data,errorMessage){
					if (errorMessage.CodeErreur == 120 || errorMessage.CodeErreur == 121 || errorMessage.CodeErreur == 122){
						window.location.hash = '#propositions?error='+errorMessage.LibelleErreur;
					} else {
						MyApp.openPopup('#dialog', errorMessage);
					}
				}
			});
		},
		
		accepter : function() {
			var thisObject = this;
			var myAppObject = MyApp;
			pamHelper.sendRestRequest({
				url: 'Mission/'+thisObject.get('Id')+'/accepter',
				type: 'POST',
				success: function (data) {
					$.mobile.hidePageLoadingMsg();
					MyApp.oSessionManager.clearCacheValue("mission");
					window.location.hash = '#propositions';
				},
				error : function (data, errorMessage){
					myAppObject.oSessionManager.clearCacheValue("mission");
					if (errorMessage.CodeErreur == 110 || errorMessage.CodeErreur == 111 || errorMessage.CodeErreur == 112){
						window.location.hash = '#propositions?error='+errorMessage.LibelleErreur;
					} else {
						myAppObject.openPopup('#dialog', errorMessage);
					}
				}
			});
		}

		
		
	});
	

	return DetailedOfferModel; 
});