/**
 * A model for availability
 */
define([
'backbone','pamhelper'
],function(Backbone,pamHelper){
	var AvailabilityModel = Backbone.Model.extend({
	    defaults: {
	    },
		dataInCache : function(idMission){
			var cachedValue = MyApp.oSessionManager.getCacheValue('planningMission_'+ idMission);
			if(cachedValue){
				console.log('data in cache');
				return JSON.parse(cachedValue);
			}
			console.log('data expired');
			return undefined;
		},
		storeDataInCache : function(idMission, data){
			MyApp.oSessionManager.setCacheValue('planningMission_'+ idMission,JSON.stringify(data));
		},
		callWebService : function(idMission){
			var thisObject = this;
			var myAppObject = MyApp;
			pamHelper.sendRestRequest({
				url:  'Planning/' + idMission,
				success: function (data) {
					thisObject.storeDataInCache(idMission, data)
					thisObject.initialiseData(thisObject, data);
				},
				error : function (data,errorMessage){ 
					window.location.hash ='#propositions?error='+errorMessage.LibelleErreur;
				}
			});
		},
		initialiseData: function(callingObject,data) {
			var thisObject = this;
			thisObject.set(data);
			setTimeout(function(){thisObject.change();console.log('change !'+thisObject);},1000);
		},
		initialize: function(){
		},
		initialise: function(idMission){
			this.idMission = idMission;
			data = this.dataInCache(idMission);
			if(data){
				this.initialiseData(this,data);
			}
			else{
				this.callWebService(idMission);
			}
		},
		commitWeek: function(callbackOnSuccess, callbackOnError){
			var thisReference = this;
			var myAppObject = MyApp;
			pamHelper.sendRestRequest({
				url: 'Planning/' + thisReference.idMission,
				type:'PUT',
				data: JSON.stringify(thisReference.attributes),
				success: function (data) {
					if(data != null && data.echec == null) {
						callbackOnSuccess(data);
					}
					else {
						var errorMessage = {CodeErreur: 83, LibelleErreur: {FR:'Attention l’un ou plusieurs des créneaux que vous avez choisis n’est plus disponible, veuillez vérifier votre planning'}};
						thisReference.set(data);
						callbackOnError(data, errorMessage);
					}
				},
				error : function (data, errorMessage){
					callbackOnError(data, errorMessage);
				}
			});
		},
		isPlanningNonAllAnswered : function (){
			for (var week = 0; week < this.attributes.Plannings.length; week++) {
				for (var day = 0; day < 7; day ++){
					for (var daytime = 0; daytime < 3; daytime ++){
						if(this.attributes.Plannings[week].Planning[day][daytime].Statut == "aRenseigner"){
							return true;
						}
					}
				}
			}
			
			return false;
		},
		hasSomeCellsAnswered : function (){
			for (var week = 0; week < this.attributes.Plannings.length; week++) {
				for (var day = 0; day < 7; day ++){
					for (var daytime = 0; daytime < 3; daytime ++){
						if(this.attributes.Plannings[week].Planning[day][daytime].Modifie == true){
							return true;
						}
					}
				}
			}
			
			return false;
		},
	    // returns the moment of the first mission day
	    // moment : instance of momentjs library
	    // getMomentDateFromISO : function that returns moment from year, week and day in week
		getFirstMissionDay: function (moment, getMomentDateFromISO) {
		    for (var day = 0; day < 7; day++) {
		        for (var daytime = 0; daytime < 3; daytime++) {
		            if (this.attributes.Plannings[0].Planning[day][daytime].Statut == "accepte" ||
							this.attributes.Plannings[0].Planning[day][daytime].Statut == "refuse" ||
							this.attributes.Plannings[0].Planning[day][daytime].Statut == "aRenseigner" ||
							this.attributes.Plannings[0].Planning[day][daytime].IdMission == this.idMission) {
		                var trueday = day + 1 == 7 ? 0 : day + 1;
		                return getMomentDateFromISO(this.attributes.Plannings[0].Annee, this.attributes.Plannings[0].Semaine, trueday);
		            }
		        }
		    }
		},
	    // returns the moment of the last mission day
	    // moment : instance of momentjs library
	    // getMomentDateFromISO : function that returns moment from year, week and day in week
		getLastMissionDay: function (moment, getMomentDateFromISO) {
		    for (var day = 6; day >= 0; day--) {
		        for (var daytime = 0; daytime < 3; daytime++) {
		            if (this.attributes.Plannings[this.attributes.Plannings.length - 1].Planning[day][daytime].Statut == "accepte" ||
							this.attributes.Plannings[this.attributes.Plannings.length - 1].Planning[day][daytime].Statut == "refuse" ||
							this.attributes.Plannings[this.attributes.Plannings.length - 1].Planning[day][daytime].Statut == "aRenseigner" ||
							this.attributes.Plannings[this.attributes.Plannings.length - 1].Planning[day][daytime].IdMission == this.idMission) {
		                var trueday = day + 1;// == 7 ? 0 : day + 1; //Je comprends pas ... on peut lui passer 0 ou 7 en tant que "dimanche", wtf ?
		                return getMomentDateFromISO(this.attributes.Plannings[this.attributes.Plannings.length - 1].Annee,
								this.attributes.Plannings[this.attributes.Plannings.length - 1].Semaine, trueday);
		            }
		        }
		    }
		},
		getMissionTitle : function() {
			for (var day = 0; day < 7; day ++){
				for (var daytime = 0; daytime < 3; daytime ++){
					if(this.attributes.Plannings[0].Planning[day][daytime].Statut == "accepte" ||
							this.attributes.Plannings[0].Planning[day][daytime].Statut == "refuse"||
							this.attributes.Plannings[0].Planning[day][daytime].Statut == "aRenseigner"||
							this.attributes.Plannings[0].Planning[day][daytime].IdMission == this.idMission){
						return this.attributes.Plannings[0].Planning[day][daytime].LibelleMission;
					}
				}
			}
		}
	});
	
	return AvailabilityModel; 
});