/**
 * Route function for the whole application
 */
define(['jquery', 'underscore', 'backbone','pamhelper'], function($, _, Backbone,pamHelper) {
	"use strict";
	var App = {};

	var Router = Backbone.Router.extend({
		routes : {
			"enCoursSynthese/:id" : "enCoursSyntheseHandler",
			"enCoursDetails/:id" : "enCoursDetailsHandler",
			"enCoursImprevu/:id" : "enCoursImprevuHandler",
			"propositionDetails/:id" : "propositionDetailsHandler",
			"propositionRefus/:id" : "propositionRefusHandler",
			"propositionRefusContrainte/:id" : "propositionRefusContrainteHandler",
			"propositionRefusMessages/:id" : "propositionRefusMessagesHandler",
			"indisponibilites/:dateDebut" : "indisponibiliteHandler",
			"mission/:id": "missionHandler",
			"changePassword/:id²": "changePasswordHandler",
			"changeEmail/:id": "changeEmailHandler",
			"acceptationPlanning/:id/:mode" : "acceptationPlanningHandler",
			"auth": "authHandler",
			"customPwd/:email" : "customPwdHandler",
			"checkUserEmail/:email": "checkUserEmailHandler",
			"checkUserEmail": "checkUserEmailHandler",
			"secretQuestion/:email/:mode": "secretQuestionHandler",
			"secretQuestion/:email/:mode/:oldpwd/:newpwd/:newlogin": "secretQuestionHandler",
			"auth/:error" : "authHandler",
			"authentificationSilencieuse/*lien" : "authentificationSilencieuse",
			"" : "defaults",
			":page?error=:error" : "generic",
			":page" : "generic"
		},
		// last argument must always be the Application
		initialize : function() {
			console.log('Router initialization...');
			// App must always be the last argument passed to the router constructor
			App = _.last(arguments);

			// Handle back button throughout the application
			$('.back').live('click', function(event) {
				window.history.back();
				return false;
			});
			this.firstPage = true;

			MyApp.openPopup = function (element_id,text)
			{
				if (text){
					var unescapeText = unescape(text);
					console.log(unescapeText);
					if (unescapeText){
						text = unescapeText;
						console.log('replace text by unescapeText');
					}
				}
				console.log(text);
				$(element_id +' div p').text(text);
				$(element_id).popup();
				$(element_id).popup('open');
				$('#closebtn').on('click', function() {
					$('body').css('overflow', 'auto');
					$('#dialog').popup('close');
				});
			}
			MyApp.openLogoutPopup = function ()
			{
				$('#dialogLogout').popup('open');	
			}
			MyApp.closeLogoutPopup = function ()
			{
				$('#dialogLogout').popup('close');	
			}
			
			console.log('Router initialized');
		},
		defaults : function() {
			console.log("View defaults");
			// Retrieve the first view
			for (var view in App.Views) {
				console.log("View : "+view);
				this.changePage(new App.Views[view], view);
				return;
			}
		},
		generic : function(id,error) {
			if (error != undefined){
				MyApp.oSessionManager.setAskedPage(id + '/' + error);
			}
			else{
				MyApp.oSessionManager.setAskedPage(id);
			}
			
			// if the asked page is auth, user does not need to be authenticated
			if(MyApp.oSessionManager.getSessionToken() == 'true' || id == 'auth'){
				console.log("View generic id : "+id+" error");
				for (var view in App.Views) {
					if (view.toLowerCase() === id.toLowerCase()) {
						console.log('routing #%s', id);
						
						var page = new App.Views[view]({message : error});
						$(page.el).attr('id', view);
						$(page.el).attr('data-role', 'page');
						// Chargement des pages pour les vues n'ayant pas de modèle propre
						if (view == 'enCours' || view == 'propositions' || view == 'auth' || view == 'planning' || view == "indisponibilites" || view == "changeEmail" || view == "changePassword") {
							this.changePage(page, id);
						}
						return;
					}
				}
				console.log('no route for #%s', id);
			}
			else{
				MyApp.oSessionManager.setErrorMessageToDisplay(MyApp.oLocale.common.error.authentificationError);
				window.location.hash = '#auth';
			}
		},
		changePage : function(page, pageid) {
			// page id	
			var id = page.id ? page.id : pageid+'-page';
			console.log("Page id :"+id);
			$(page.el).attr('id', id);
			
			$(page.el).attr('data-role', 'page');
			page.render();
			$('body').append($(page.el));
			$.mobile.defaultPageTransition = 'none';
			$.mobile.changePage.defaults.transition = 'none';
			// specific jqm page options
			var transition = page.transition ? page.transition : $.mobile.defaultPageTransition ;
			var reverse = MyApp.reversePage ? MyApp.reversePage : false;
			// We don't want to slide the first page
			if (MyApp.notFirstPage) {
				//transition = 'slide';
				MyApp.notFirstPage = true;
				if(reverse){
					transition = 'none';
				}
			}else{
				transition = 'none';
				MyApp.notFirstPage = true;
			}
			
			console.log(transition);
			
			$.mobile.changePage($(page.el), {
				changeHash : false,
				transition : transition,
				reverse : reverse
			});
			MyApp.reversePage = false;
			$.mobile.silentScroll(0);
		},
		authHandler : function (error) {
			//Adds a enCoursSynthese view;
			console.log('authHandler :'+error);
			var page = new App.Views['auth']({message : error});
			
			$(page.el).attr('id', error);
			$(page.el).attr('data-role', 'page');
			page.showMessage();
		},
		customPwdHandler: function (email) {
		    console.log('customPwd :' + email);
		    var page = new App.Views['customPwd']({ email: email });

		    $(page.el).attr('id', email);
		    $(page.el).attr('data-role', 'page');
		    //page.showMessage();
		},
		checkUserEmailHandler: function (email) {
            if(email)
                MyApp.oSessionManager.setAskedPage('checkUserEmail/' + email);
            else
                MyApp.oSessionManager.setAskedPage('checkUserEmail');

		    console.log('checkUserEmailHandler :' + email);
		    var page = new App.Views['checkUserEmail']({ email: email });

		    $(page.el).attr('id', email);
		    $(page.el).attr('data-role', 'page');
		    //page.showMessage();
		},
		changePasswordHandler: function (id) {
		    MyApp.oSessionManager.setAskedPage('changePassword');

		    console.log('changePassword :' + id);
		    var page = new App.Views['changePassword']({ id: id });

		    $(page.el).attr('id', 'changePassword');
		    $(page.el).attr('data-role', 'page');
		},
		changeEmailHandler: function (id) {
		    MyApp.oSessionManager.setAskedPage('changeEmail');

		    console.log('changeEmail :' + id);
		    var page = new App.Views['changeEmail']({ id: id });

		    $(page.el).attr('id', 'changePassword');
		    $(page.el).attr('data-role', 'page');
		},
		secretQuestionHandler : function(email, mode,oldpwd,newpwd,newlogin){
		    if (mode)
		        MyApp.oSessionManager.setAskedPage('secretQuestion/' + email + '/' + mode + '/' + oldpwd + '/' + newpwd + '/' + newlogin );
		    else
		        MyApp.oSessionManager.setAskedPage('secretQuestion' + email);
		    console.log('secretQuestionHandler :' + email);
		    var page = new App.Views['secretQuestion']({ email: email, mode:mode, oldpwd:oldpwd, newpwd:newpwd, newlogin:newlogin});

		    $(page.el).attr('id', email);
		    $(page.el).attr('data-role', 'page');
		},
		enCoursSyntheseHandler : function (id) {
			MyApp.oSessionManager.setAskedPage('enCoursSynthese/' + id);
			if(MyApp.oSessionManager.getSessionToken() == 'true'){
				//Adds a enCoursSynthese view;
				var page = new App.Views['enCoursSynthese']({idMission : id});
				$(page.el).attr('id', id);
				$(page.el).attr('data-role', 'page');
			}
			else{
				MyApp.oSessionManager.setErrorMessageToDisplay(MyApp.oLocale.common.error.authentificationError);
				window.location.hash = '#auth';
			}
		},
		enCoursDetailsHandler : function (id) {
			MyApp.oSessionManager.setAskedPage('enCoursDetails/' + id);
			if(MyApp.oSessionManager.getSessionToken() == 'true'){
				//Adds a enCoursDetails view;
				var page = new App.Views['enCoursDetails']({idMission : id});
				$(page.el).attr('id', id);
				$(page.el).attr('data-role', 'page');
			}
			else{
				MyApp.oSessionManager.setErrorMessageToDisplay(MyApp.oLocale.common.error.authentificationError);
				window.location.hash = '#auth';
			}
		},
		enCoursImprevuHandler : function (id) {
			MyApp.oSessionManager.setAskedPage('enCoursImprevu/' + id);
			if(MyApp.oSessionManager.getSessionToken() == 'true'){
				//Adds a enCoursDetails view;
				var page = new App.Views['enCoursImprevu']({idMission : id});
				$(page.el).attr('id', id);
				$(page.el).attr('data-role', 'page');
			}
			else{
				MyApp.oSessionManager.setErrorMessageToDisplay(MyApp.oLocale.common.error.authentificationError);
				window.location.hash = '#auth';
			}
		},
		propositionDetailsHandler : function (id) {
			MyApp.oSessionManager.setAskedPage('propositionDetails/' + id);
			if(MyApp.oSessionManager.getSessionToken() == 'true'){
				//Adds a propositionDetails view;
				var page = new App.Views['propositionDetails']({idMission : id});
				$(page.el).attr('id', id);
				$(page.el).attr('data-role', 'page');
			}
			else{
				MyApp.oSessionManager.setErrorMessageToDisplay(MyApp.oLocale.common.error.authentificationError);
				window.location.hash = '#auth';
			}
		},
		propositionRefusHandler : function (id) {
			MyApp.oSessionManager.setAskedPage('propositionRefus/' + id);
			if(MyApp.oSessionManager.getSessionToken() == 'true'){
				//Adds a propositionRefus view;
				var page = new App.Views['propositionRefus']({idMission : id});
				$(page.el).attr('id', id);
				$(page.el).attr('data-role', 'page');
			}
			else{
				MyApp.oSessionManager.setErrorMessageToDisplay(MyApp.oLocale.common.error.authentificationError);
				window.location.hash = '#auth';
			}
		},
		propositionRefusContrainteHandler : function (id) {
			MyApp.oSessionManager.setAskedPage('propositionRefusContrainte/' + id);
			if(MyApp.oSessionManager.getSessionToken() == 'true'){
				//Adds a propositionRefus view;
				var page = new App.Views['propositionRefusContrainte']({idMission : id});
				$(page.el).attr('id', id);
				$(page.el).attr('data-role', 'page');
			}
			else{
				MyApp.oSessionManager.setErrorMessageToDisplay(MyApp.oLocale.common.error.authentificationError);
				window.location.hash = '#auth';
			}
		},
		propositionRefusMessagesHandler : function (id) {
			MyApp.oSessionManager.setAskedPage('propositionRefusMessages/' + id);
			if(MyApp.oSessionManager.getSessionToken() == 'true'){
				//Adds a propositionRefus view;
				var page = new App.Views['propositionRefusMessages']({idMission : id});
				$(page.el).attr('id', id);
				$(page.el).attr('data-role', 'page');
			}
			else{
				MyApp.oSessionManager.setErrorMessageToDisplay(MyApp.oLocale.common.error.authentificationError);
				window.location.hash = '#auth';
			}
		},
		// prise en compte du mode d'accès : readOnly ou readWrite
		acceptationPlanningHandler : function (id, mode){
			MyApp.oSessionManager.setAskedPage('acceptationPlanning/' + id + '/' + mode);
			if(MyApp.oSessionManager.getSessionToken() == 'true'){
				//Adds an acceptationPlanning view
				var page = new App.Views['acceptationPlanning']({idMission : id, accessMode : mode});
				$(page.el).attr('id', 'planning' + id);
				$(page.el).attr('data-role', 'page');
			}
			else{
				MyApp.oSessionManager.setErrorMessageToDisplay(MyApp.oLocale.common.error.authentificationError);
				window.location.hash = '#auth';
			}
		},
		indisponibiliteHandler : function (dateDebut){
			MyApp.oSessionManager.setAskedPage('indisponibilites/' + dateDebut);
			if(MyApp.oSessionManager.getSessionToken() == 'true'){
				var page = new App.Views['indisponibilites']({dateDebutMissionRefusee : dateDebut});
				$(page.el).attr('dateDebut', 'planning' + dateDebut);
				$(page.el).attr('data-role', 'page');
			}
			else{
				MyApp.oSessionManager.setErrorMessageToDisplay(MyApp.oLocale.common.error.authentificationError);
				window.location.hash = '#auth';
			}
		},
		missionHandler : function (id) {
			//Adds a propositionRefus view;
			//Recherche de la mission dans le cache
			
			MyApp.oSessionManager.setAskedPage('mission/' + id);
			if(MyApp.oSessionManager.getSessionToken() == 'true'){
				var cachedValue = MyApp.oSessionManager.getCacheValue('details_'+id);
				var missionDetails;
				if(cachedValue){
					this.changeMissionPage(id);
				}
				// si les données ne sont pas cachées, on les télécharge
				var thisObject = this;
				var myAppObject = MyApp;
				pamHelper.sendRestRequest({
					url: 'Mission/' + id+'/Details',
					success: function (data) {
						MyApp.oSessionManager.setCacheValue('details_'+id, JSON.stringify(data));
						thisObject.changeMissionPage(id);
					},
					error : function (data,errorMessage){
						window.location.hash ='#propositions?error='+errorMessage.LibelleErreur;						
					}
				});
			} else {
				MyApp.oSessionManager.setErrorMessageToDisplay(MyApp.oLocale.common.error.authentificationError);
				window.location.hash = '#auth';
			}
		},
		changeMissionPage : function (id) {
			var cachedValue = MyApp.oSessionManager.getCacheValue('details_'+id);
			var mission = JSON.parse(cachedValue);
			if (mission.Statut == 'en cours'){
				var page = new App.Views['enCoursSynthese']({idMission : id});
				$(page.el).attr('id', id);
				$(page.el).attr('data-role', 'page');
			}
			else {
				var page = new App.Views['propositionDetails']({idMission : id});
				$(page.el).attr('id', id);
				$(page.el).attr('data-role', 'page');
			}
		},

		authentificationSilencieuse : function (lien) {
			if(MyApp.oSessionManager.getToken() && MyApp.oSessionManager.getToken()!= "") 
			{
				pamHelper.sendRestRequest({
					url: 'authpam/Authentification/auth',
					type:'POST',
					data: JSON.stringify({DeviceID: MyApp.oSessionManager.getDeviceToken(), DeviceName: MyApp.oConfig.deviceName(), VersionAppli: MyApp.oConfig.appVersion() ,CodeMarque: MyApp.oConfig.marque(), DeviceFamily: MyApp.oConfig.deviceFamily()}),
					success : function(data) {
						if(lien != '') {
							window.location.hash = lien;
						}
					}
				});
			} else {
				if(lien != '') {
					window.location.hash = lien;
				}	
			}

		}
	});

	return Router;
});
